Vampire
==============

By Richard Evans

Vampire is a high performance general purpose code for the atomistic simulation of magnetic materials. Using a variety of common simulation methods it can calculate the equilibrium and dynamic magnetic properties of a wide variety of magnetic materials and phenomena, including ferro, ferri and antiferromagnets, core-shell nanoparticles, ultrafast spin dynamics, magnetic recording media, heat assisted magnetic recording, exchange bias, magnetic multilayer films and complete devices.

Capabilities
---------------
Vampire is designed to be highly flexible to deal with a wide variety of problems using a diverse set of simulation tools and methods. The capabilities of the code can be summarised broadly in terms of the simulation methods, standard problems, structural properties and features of the code, all of which can be combined to tackle almost any problem.

**Simulation methods**
-Stochastic Landau-Lifshitz-Gilbert equation (spin dynamics)
-Monte Carlo metropolis
-Constrained Monte Carlo metropolis

**Standard calculations**
-Ultrafast spin dynamics
-Hysteresis loops
-Curie temperature
-Temperature dependent anisotropy
-Temperature dependent energy barriers
-Field cooling
-Heat assisted and conventional magnetic recording
-Laser induced spin dynamics

**Structural properties**
-Bulk-like systems
-Thin films
-Nanoparticles - spheres, cubes, truncated octahedra, cylinders
-Voronoi granular structures
-Nanoparticle arrays
-Core-shell nanoparticles
-Multilayer thin films
-Interface roughness and intermixing
-Dilute magnetic systems
-Lithographically defined geometries
-SC, FCC, HCP, and BCC crystal structures
-User-defined atomic structures - for example from Molecular Dynamics simulations

**Magnetic properties**
-Ferromagnets
-Antiferromagnets
-Ferrimagnets
-Spin glass
-Single-ion, 2-ion and cubic anisotropies
-Scalar, vector and tensor forms of exchange including the DM interaction
-User-defined Hamiltonian from ab-initio Density Functional Theory (DFT) calculations
-Demagnetisation fields (macrocell approximation)

**Code features**
-Modular object-oriented C++
-Simple to use textfile input
-High performance code
-Parallelisation using the MPI library
-Variety of geometric decomposition algorithms
-Usable on a laptop to a supercomputer with thousands of cores
-Output to PoVRAY for visualisation and publication quality graphics
-Output to rasmol/jmol for structural inspection
-Minimal dependence on external libraries for portability
-Freely available open source code

License
---------------
See the license file.

